package com.diagbot.facade;

import com.diagbot.client.TranServiceClient;
import com.diagbot.client.bean.HosCodeVO;
import com.diagbot.dto.LisResult;
import com.diagbot.dto.PacsResult;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.SysSetInfoDTO;
import com.diagbot.enums.LisSourceEnum;
import com.diagbot.enums.SysTypeEnum;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.DiseaseIcdVO;
import com.diagbot.vo.HospitalSetVO;
import com.diagbot.vo.LisConfigVO;
import com.diagbot.vo.PacsConfigVO;
import com.google.common.collect.Lists;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:
 * @Author:zhaops
 * @time: 2019/11/11 14:14
 */
@Component
public class TransferFacade {
    @Autowired
    private TranServiceClient tranServiceClient;


    /**
     * 查询对接状态
     *
     * @param hosCode
     * @param sysType
     * @return
     */
    public Boolean isConnect(String hosCode, Integer sysType) {
        Boolean isConnect = false;
        if (StringUtil.isNotBlank(hosCode)) {
            HosCodeVO hosCodeVO = new HosCodeVO();
            hosCodeVO.setHosCode(hosCode);
            HospitalSetVO hospitalSetVO = new HospitalSetVO();
            hospitalSetVO.setHospitalCode(hosCode);
            hospitalSetVO.setCode("connect");
            if (sysType != null) {
                hospitalSetVO.setSysType(sysType);
            } else {
                hospitalSetVO.setSysType(SysTypeEnum.AIPT_SERVICE.getKey());
            }
            RespDTO<List<SysSetInfoDTO>> sysSetInfoListRes = tranServiceClient.getSysSetInfoDatas(hospitalSetVO);
            if (RespDTOUtil.respIsOK(sysSetInfoListRes)) {
                List<SysSetInfoDTO> sysSetInfoList = sysSetInfoListRes.data;
                if (ListUtil.isNotEmpty(sysSetInfoList)) {
                    if (sysSetInfoList.get(0).getValue().equals(1)) {
                        isConnect = true;
                    } else {
                        isConnect = false;
                    }
                } else {
                    isConnect = false;
                }
            }
        }
        return isConnect;
    }

    /**
     * 增加化验公表项
     *
     * @param lisResults
     * @param hosCode
     * @return
     */
    public List<LisResult> addLisUniqueName(List<LisResult> lisResults, String hosCode) {
        //化验项转公表内容,参数处理
        if (ListUtil.isNotEmpty(lisResults)) {
            List<String> mealNameList = lisResults
                    .stream()
                    .map(lisResult -> lisResult.getName()).collect(Collectors.toList());
            LisConfigVO lisConfigVO = new LisConfigVO();
            lisConfigVO.setHosCode(hosCode);
            lisConfigVO.setMealNameList(mealNameList);
            RespDTO<Map<String, Map<String, String>>> lisConfigRes
                    = tranServiceClient.getLisConfigByMealNameAndHosCode(lisConfigVO);
            Map<String, Map<String, String>> lisConfigMap = new LinkedHashMap<>();
            if (RespDTOUtil.respIsOK(lisConfigRes)) {
                lisConfigMap = lisConfigRes.data;
            }
            for (LisResult lisResult : lisResults) {
                if (!lisResult.getSource().equals(LisSourceEnum.Outer.getKey())) {
                    continue;
                }
                if (lisResult.getDetailName() == null) {
                    lisResult.setDetailName("");
                }
                if (lisConfigMap.get(lisResult.getName()) != null) {
                    lisResult.setUniqueName(lisConfigMap.get(lisResult.getName()).get(lisResult.getDetailName()));
                }
            }
        }
        return lisResults;
    }

    /**
     * 获取化验公表映射关系
     *
     * @param nameList
     * @param hosCode
     * @return
     */
    public Map<String, Map<String, String>> getLisConfigByMealNameAndHosCode(List<String> nameList, String hosCode) {
        Map<String, Map<String, String>> lisConfigMap = new LinkedHashMap<>();
        if (ListUtil.isNotEmpty(nameList) && StringUtil.isNotBlank(hosCode)) {
            LisConfigVO lisConfigVO = new LisConfigVO();
            lisConfigVO.setHosCode(hosCode);
            lisConfigVO.setMealNameList(nameList);
            RespDTO<Map<String, Map<String, String>>> lisConfigRes
                    = tranServiceClient.getLisConfigByMealNameAndHosCode(lisConfigVO);
            if (RespDTOUtil.respIsOK(lisConfigRes)) {
                lisConfigMap = lisConfigRes.data;
            }
        }
        return lisConfigMap;
    }

    /**
     * 辅检公表转换
     *
     * @param pacsArr
     * @param hosCode
     * @return
     */
    public List<PacsResult> addPacsUniqueName(List<PacsResult> pacsArr, String hosCode) {
        Map<String, PacsResult> pacsOriginalMap = EntityUtil.makeEntityMap(pacsArr, "name");
        List<PacsResult> convertPacsArr = Lists.newLinkedList();
        PacsConfigVO pacsConfigVO = new PacsConfigVO();
        pacsConfigVO.setHosCode(hosCode);
        pacsConfigVO.setMealNameList(pacsArr
                .stream()
                .map(pacsResult -> pacsResult.getName())
                .distinct()
                .collect(Collectors.toList()));
        RespDTO<Map<String, List<String>>> pacsRespDTO = tranServiceClient.getPacsConfig(pacsConfigVO);
        if (RespDTOUtil.respIsOK(pacsRespDTO)) {
            Map<String, List<String>> pacsMap = pacsRespDTO.data;
            for (Map.Entry<String, PacsResult> entry : pacsOriginalMap.entrySet()) {
                if (ListUtil.isNotEmpty(pacsMap.get(entry.getKey()))) {
                    for (String uniqueName : pacsMap.get(entry.getKey())) {
                        PacsResult pacsResult = new PacsResult();
                        BeanUtil.copyProperties(entry.getValue(), pacsResult);
                        pacsResult.setUniqueName(uniqueName);
                        convertPacsArr.add(pacsResult);
                    }
                } else {
                    convertPacsArr.add(entry.getValue());
                }
            }
        }
        return convertPacsArr;
    }

    /**
     * 诊断名称映射转换
     *
     * @param diseaseStr
     * @param hosCode
     * @param type       1-外部名称转内部名称，2-内部名称转外部名称
     * @return
     */
    public String convertDisease(String diseaseStr, String hosCode, Integer type) {
        String[] diseases = diseaseStr.replace(" ", "").split(",|，|;|；");
        List<String> diseaseList = Arrays.asList(diseases);
        diseaseList = diseaseList
                .stream()
                .filter(i -> StringUtil.isNotBlank(i))
                .distinct()
                .collect(Collectors.toList());
        DiseaseIcdVO diseaseIcdVO = new DiseaseIcdVO();
        diseaseIcdVO.setHosCode(hosCode);
        if (type.equals(1)) {
            diseaseIcdVO.setDiseaseNameList(diseaseList);
        } else if (type.equals(2)) {
            diseaseIcdVO.setConceptDisNameList(diseaseList);
        }
        String convertDiag = "";
        Map<String, String> disMap = new LinkedHashMap<>();
        RespDTO<Map<String, String>> disRespDTO = tranServiceClient.getDiseaseIcdMap(diseaseIcdVO);
        if (RespDTOUtil.respIsOK(disRespDTO)) {
            disMap = disRespDTO.data;
            List<String> disNameList = Lists.newLinkedList();
            if (type.equals(1)) {
                //外部名称转内部名称
                disNameList = diseaseIcdVO.getDiseaseNameList();
            } else if (type.equals(2)) {
                //内部名称转外部名称
                disNameList = diseaseIcdVO.getConceptDisNameList();
            }

            for (String diseaseName : disNameList) {
                if (StringUtil.isNotBlank(disMap.get(diseaseName))) {
                    convertDiag += disMap.get(diseaseName) + ",";
                }
            }
        }
        return convertDiag;
    }
}
